@@ -1,6 +1,7 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  *  Authors: Jeffrey Stedfast <fejj@ximian.com>
+ *           Michael Zucchi <notzed@ximian.com>
  *
  *  Copyright 2001-2004 Ximian, Inc. (www.ximian.com)
  *
@@ -121,6 +122,7 @@ static struct {
 	{ "gbk",         "zh" },
 	{ "euc-tw",      "zh" },
 	{ "iso-2022-jp", "ja" },
+	{ "Shift-JIS",   "ja" },
 	{ "sjis",        "ja" },
 	{ "ujis",        "ja" },
 	{ "eucJP",       "ja" },
@@ -130,9 +132,6 @@ static struct {
 	{ "koi8-u",      "uk" }
 };
 
-#define NUM_CJKR_LANGS (sizeof (cjkr_lang_map) / sizeof (cjkr_lang_map[0]))
-
-
 static GHashTable *iconv_charsets = NULL;
 static char *locale_charset = NULL;
 static char *locale_lang = NULL;
@@ -158,7 +157,7 @@ iconv_charset_free (gpointer key, gpointer value, gpointer user_data)
 /**
  * g_mime_charset_map_shutdown:
  *
- * Frees internal lookup tables created in #g_mime_charset_map_init().
+ * Frees internal lookup tables created in g_mime_charset_map_init().
  **/
 void
 g_mime_charset_map_shutdown (void)
@@ -192,7 +191,8 @@ locale_parse_lang (const char *locale)
 	if (strlen (lang) >= 2) {
 		if (lang[2] == '-' || lang[2] == '_') {
 			/* canonicalise the lang */
-			g_ascii_strdown (lang, 2);
+			lang[0] = g_ascii_tolower (lang[0]);
+			lang[1] = g_ascii_tolower (lang[1]);
 			
 			/* validate the country code */
 			if (strlen (lang + 3) > 2) {
@@ -200,7 +200,8 @@ locale_parse_lang (const char *locale)
 				lang[2] = '\0';
 			} else {
 				lang[2] = '-';
-				g_ascii_strup (lang + 3, 2);
+				lang[3] = g_ascii_toupper (lang[3]);
+				lang[4] = g_ascii_toupper (lang[4]);
 			}
 		} else if (lang[2] != '\0') {
 			/* invalid language */
@@ -221,7 +222,7 @@ locale_parse_lang (const char *locale)
  * g_mime_charset_map_init:
  *
  * Initializes the locale charset variable for later calls to
- * #g_mime_locale_charset(). Only really needs to be called for non-
+ * g_mime_locale_charset(). Only really needs to be called for non-
  * iso-8859-1 locales.
  **/
 void
@@ -236,16 +237,18 @@ g_mime_charset_map_init (void)
 	iconv_charsets = g_hash_table_new (g_str_hash, g_str_equal);
 	
 	for (i = 0; known_iconv_charsets[i].charset != NULL; i++) {
-		charset = g_strdup (known_iconv_charsets[i].charset);
+		charset = g_ascii_strdown (known_iconv_charsets[i].charset, -1);
 		iconv_name = g_strdup (known_iconv_charsets[i].iconv_name);
-		g_ascii_strdown (charset, -1);
 		g_hash_table_insert (iconv_charsets, charset, iconv_name);
 	}
 	
 #ifdef HAVE_CODESET
-	locale_charset = g_strdup (nl_langinfo (CODESET));
-	g_ascii_strdown (locale_charset, -1);
-#else
+	if ((locale_charset = nl_langinfo (CODESET)) && locale_charset[0])
+		locale_charset = g_ascii_strdown (locale_charset, -1);
+	else
+		locale_charset = NULL;
+#endif
+	
 	locale = setlocale (LC_ALL, NULL);
 	
 	if (!locale || !strcmp (locale, "C") || !strcmp (locale, "POSIX")) {
@@ -264,22 +267,25 @@ g_mime_charset_map_init (void)
 		 */
 		char *codeset, *p;
 		
-		codeset = strchr (locale, '.');
-		if (codeset) {
-			codeset++;
-			
-			/* ; is a hack for debian systems and / is a hack for Solaris systems */
-			for (p = codeset; *p && !strchr ("@;/", *p); p++);
-			locale_charset = g_strndup (codeset, (unsigned) (p - codeset));
-			g_ascii_strdown (locale_charset, -1);
-		} else {
-			/* charset unknown */
-			locale_charset = NULL;
+		if (!locale_charset) {
+			codeset = strchr (locale, '.');
+			if (codeset) {
+				codeset++;
+				
+				/* ; is a hack for debian systems and / is a hack for Solaris systems */
+				p = codeset;
+				while (*p && !strchr ("@;/", *p))
+					p++;
+				
+				locale_charset = g_ascii_strdown (codeset, p - codeset);
+			} else {
+				/* charset unknown */
+				locale_charset = NULL;
+			}
 		}
 		
 		locale_parse_lang (locale);
 	}
-#endif
 }
 
 
@@ -340,7 +346,7 @@ g_mime_charset_language (const char *charset)
 	if (!charset)
 		return NULL;
 	
-	for (i = 0; i < NUM_CJKR_LANGS; i++) {
+	for (i = 0; i < G_N_ELEMENTS (cjkr_lang_map); i++) {
 		if (!strcasecmp (cjkr_lang_map[i].charset, charset))
 			return cjkr_lang_map[i].lang;
 	}
@@ -477,8 +483,6 @@ static const char *iso_charsets[] = {
 	"iso-8859-16"
 };
 
-#define NUM_ISO_CHARSETS (sizeof (iso_charsets) / sizeof (iso_charsets[0]))
-
 static const char *windows_charsets[] = {
 	"windows-cp1250",
 	"windows-cp1251",
@@ -500,7 +504,7 @@ static const char *windows_charsets[] = {
  * Attempts to find a canonical charset name for @charset.
  *
  * Note: Will normally return the same value as
- * #g_mime_charset_iconv_name() unless the system iconv does not use
+ * g_mime_charset_iconv_name() unless the system iconv does not use
  * the canonical ISO charset names (such as using ISO8859-1 rather
  * than the canonical form ISO-8859-1).
  *
@@ -533,7 +537,7 @@ g_mime_charset_canon_name (const char *charset)
 		if (endptr == ptr || *endptr != '\0')
 			return charset;
 		
-		if (iso >= NUM_ISO_CHARSETS)
+		if (iso > G_N_ELEMENTS (iso_charsets))
 			return charset;
 		
 		return iso_charsets[iso];
@@ -553,7 +557,7 @@ g_mime_charset_canon_name (const char *charset)
  *
  * Attempts to find an iconv-friendly charset name for @charset.
  *
- * Note: This function is deprecated. Use #g_mime_charset_iconv_name
+ * Note: This function is deprecated. Use g_mime_charset_iconv_name()
  * instead.
  *
  * Returns an iconv-friendly charset name for @charset.
@@ -570,7 +574,7 @@ g_mime_charset_name (const char *charset)
  *
  * Gets the user's locale charset (or iso-8859-1 by default).
  *
- * Note: This function is deprecated. Use #g_mime_locale_charset
+ * Note: This function is deprecated. Use g_mime_locale_charset()
  * instead.
  *
  * Returns the user's locale charset (or iso-8859-1 by default).
@@ -659,7 +663,7 @@ g_mime_charset_init (GMimeCharset *charset)
 
 /**
  * g_mime_charset_step:
- * @charset:
+ * @charset: charset structure
  * @in: input text buffer (must be in UTF-8)
  * @len: input buffer length
  *
@@ -711,11 +715,16 @@ g_mime_charset_step (GMimeCharset *charset, const char *in, size_t len)
 static const char *
 charset_best_mask (unsigned int mask)
 {
+	const char *lang;
 	int i;
 	
-	for (i = 0; i < sizeof (charinfo) / sizeof (charinfo[0]); i++) {
-		if (charinfo[i].bit & mask)
-			return charinfo[i].name;
+	for (i = 0; i < G_N_ELEMENTS (charinfo); i++) {
+		if (charinfo[i].bit & mask) {
+			lang = g_mime_charset_language (charinfo[i].name);
+			
+			if (!lang || (locale_lang && !strncmp (locale_lang, lang, 2)))
+				return charinfo[i].name;
+		}
 	}
 	
 	return "UTF-8";
