@@ -28,6 +28,7 @@
 #include <string.h>
 
 #include "gmime-multipart-signed.h"
+#include "gmime-multipart-encrypted.h"
 #include "gmime-message-part.h"
 #include "gmime-stream-filter.h"
 #include "gmime-filter-strip.h"
@@ -222,6 +223,11 @@ sign_prepare (GMimeObject *mime_part)
 	if (GMIME_IS_MULTIPART (mime_part)) {
 		GList *lpart;
 		
+		if (GMIME_IS_MULTIPART_SIGNED (mime_part) || GMIME_IS_MULTIPART_ENCRYPTED (mime_part)) {
+			/* must not modify these parts as they must be treated as opaque */
+			return;
+		}
+		
 		lpart = GMIME_MULTIPART (mime_part)->subparts;
 		while (lpart) {
 			subpart = GMIME_OBJECT (lpart->data);
