@@ -61,6 +61,7 @@ test_parser (GMimeStream *stream)
 	parser = g_mime_parser_new ();
 	g_mime_parser_init_with_stream (parser, stream);
 	g_mime_parser_set_scan_from (parser, TRUE);
+	/*g_mime_parser_set_respect_content_length (parser, TRUE);*/
 	
 	g_mime_parser_set_header_regex (parser, "^X-Evolution$", header_cb, NULL);
 	
@@ -90,8 +91,6 @@ int main (int argc, char **argv)
 	GMimeStream *stream;
 	int fd;
 	
-	g_mime_init (0);
-	
 	if (argc > 1)
 		filename = argv[1];
 	else
@@ -101,6 +100,8 @@ int main (int argc, char **argv)
 	if (fd == -1)
 		return 0;
 	
+	g_mime_init (0);
+	
 #ifdef STREAM_MMAP
 	stream = g_mime_stream_mmap_new (fd, PROT_READ, MAP_PRIVATE);
 	g_assert (stream != NULL);
@@ -127,5 +128,7 @@ int main (int argc, char **argv)
 	
 	g_object_unref (stream);
 	
+	g_mime_shutdown ();
+	
 	return 0;
 }
